import java.util.Stack;

public class CollectBot extends SmartBot {

    private int maxNumberOfTilesToMove = 15;

    private int tilesMoved;

    private char directionBias;

    private final Stack<Character> moves;

    protected CollectBot(String[] args) {
        super(args);
        tilesMoved = 0;
        directionBias = LEFT;
        moves = new Stack<Character>();
    }

    public static void main(String[] args) {
        Bot bot = new CollectBot(args);
        bot.run();
    }

    @Override
    protected char nextMove(View view) throws Exception {
        char move = '^';

        // Search larger areas after a time
        if (moves.size() != 0 && moves.size() % 100 == 0 && maxNumberOfTilesToMove <=30) {
            maxNumberOfTilesToMove += 5;
        }

        // Look for Gems
        int turn = scanView(view);
        if (turn == 3) {
            tilesMoved++;
            // Change up direction after not encountering anything for a while
            if (tilesMoved >= maxNumberOfTilesToMove) {
                tilesMoved = 0;
                move = switchBias(true);
            }
        } else {
            tilesMoved = 0;
            move = switch (turn) {
                case -1 -> LEFT;
                case 0 -> FORWARD;
                case 1 -> RIGHT;
                default -> move;
            };
        }
        moves.push(move);
        return move;
    }

    /***
     * This Method can be either used to detect potential spiral collision paths or to periodically change the direction bias
     * @param numberOfTurns The number of turns that should be equal
     * @return true, if the last n turns were equal in the given distance covered
     */
    private boolean evaluateNumberOfEqualTurns(int numberOfTurns) {
        if (this.moves.isEmpty() || this.moves.size() < 3) {
            return false;
        }
        Stack<Character> moves = (Stack<Character>) this.moves.clone();
        int numberOfEqualTurns = 0;
        int distanceCovered = 0;
        char directionToCompare = '#';
        while (numberOfEqualTurns <= numberOfTurns && !moves.isEmpty()) {
            char lastMove = moves.pop();
            if (lastMove == FORWARD) {
                distanceCovered++;
                continue;
            }

            if (numberOfEqualTurns == 0) {
                numberOfEqualTurns++;
                directionToCompare = lastMove;
            } else {
                if (lastMove != directionToCompare) {
                    return false;
                }
                numberOfEqualTurns++;
            }
        }
        return numberOfEqualTurns == 3;
    }

    /***
     * Changes up the direction bias every two turns which achieves a rough zig-zag pattern
     * @return the next direction bias
     */
    protected char switchBias(boolean forcedSwitch) {
        if(!evaluateNumberOfEqualTurns(2) && !forcedSwitch) {
            return directionBias;
        }
        directionBias = switch (directionBias) {
            case LEFT -> {
                directionBias = RIGHT;
                yield RIGHT;
            }
            case RIGHT -> {
                directionBias = LEFT;
                yield LEFT;
            }
            default -> directionBias;
        };
        return directionBias;
    }

    protected char switchBias() {
        return switchBias(false);
    }

    protected int scanView(View view) {
        if (view.data.contains("@")) {
            int stone = view.data.indexOf("@") + 1;

            // Check Front:
            switch (stone) {
                case 3, 8:
                    return 0;
            }

            // If possible, prefer direction Bias
            if (directionBias == LEFT) {
                switch (stone) {
                    case 11, 12: return -1; // left
                    case 14, 15: return  1; // right
                }
            } else {
                switch (stone) {
                    case 14, 15: return  1; // right
                    case 11, 12: return -1; // left
                }
            }
        }
        return 3;
    }
}
